@@ -73,11 +73,19 @@ typedef const char *hdr_t;
 
 /*--------------------------------------------------------------------*/
 
-enum sess_close {
-	SC_NULL = 0,
-#define SESS_CLOSE(nm, stat, err, desc)	SC_##nm,
-#include "tbl/sess_close.h"
+struct stream_close {
+	unsigned		magic;
+#define STREAM_CLOSE_MAGIC	0xc879c93d
+	int			idx;
+	unsigned		is_err;
+	const char		*name;
+	const char		*desc;
 };
+    extern const struct stream_close SC_NULL[1];
+#define SESS_CLOSE(nm, stat, err, desc) \
+    extern const struct stream_close SC_##nm[1];
+#include "tbl/sess_close.h"
+
 
 /*--------------------------------------------------------------------
  * Indices into http->hd[]
@@ -198,7 +206,7 @@ struct pool_task {
  * TASK_QUEUE_RUSH is req's returning from waiting list
  *
  * NOTE: When changing the number of classes, update places marked with
- * TASK_QUEUE__END in mgt_pool.c
+ * TASK_QUEUE_RESERVE in params.h
  */
 enum task_prio {
 	TASK_QUEUE_BO,
@@ -206,11 +214,13 @@ enum task_prio {
 	TASK_QUEUE_REQ,
 	TASK_QUEUE_STR,
 	TASK_QUEUE_VCA,
+	TASK_QUEUE_BG,
 	TASK_QUEUE__END
 };
 
 #define TASK_QUEUE_HIGHEST_PRIORITY TASK_QUEUE_BO
-#define TASK_QUEUE_CLIENT(prio) \
+#define TASK_QUEUE_RESERVE TASK_QUEUE_BG
+#define TASK_QUEUE_LIMITED(prio) \
 	(prio == TASK_QUEUE_REQ || prio == TASK_QUEUE_STR)
 
 /*--------------------------------------------------------------------*/
@@ -236,7 +246,8 @@ struct worker {
 
 	unsigned		cur_method;
 	unsigned		seen_methods;
-	unsigned		handling;
+
+	struct wrk_vpi		*vpi;
 };
 
 /* Stored object -----------------------------------------------------
@@ -383,8 +394,10 @@ struct busyobj {
 
 	struct pool_task	fetch_task[1];
 
-#define BO_FLAG(l, r, rr, rw, f, d) unsigned	l:1;
-#include "tbl/bo_flags.h"
+#define BERESP_FLAG(l, r, w, f, d) unsigned	l:1;
+#define BEREQ_FLAG(l, r, w, d) BERESP_FLAG(l, r, w, 0, d)
+#include "tbl/bereq_flags.h"
+#include "tbl/beresp_flags.h"
 
 	/* Timeouts */
 	vtim_dur		connect_timeout;
@@ -393,6 +406,7 @@ struct busyobj {
 
 	/* Timers */
 	vtim_real		t_first;	/* First timestamp logged */
+	vtim_real		t_resp;		/* response received */
 	vtim_real		t_prev;		/* Previous timestamp logged */
 
 	/* Acct */
@@ -431,7 +445,7 @@ struct req {
 #define REQ_MAGIC		0xfb4abf6d
 
 	body_status_t		req_body_status;
-	enum sess_close		doclose;
+	stream_close_t		doclose;
 	unsigned		restarts;
 	unsigned		esi_level;
 
@@ -484,6 +498,7 @@ struct req {
 	vtim_real		t_first;	/* First timestamp logged */
 	vtim_real		t_prev;		/* Previous timestamp logged */
 	vtim_real		t_req;		/* Headers complete */
+	vtim_real		t_resp;		/* Entry to last deliver/synth */
 
 	struct http_conn	*htc;
 	struct vfp_ctx		*vfc;
@@ -583,6 +598,7 @@ void HTTP_Clone(struct http *to, const struct http * const fm);
 void HTTP_Dup(struct http *to, const struct http * const fm);
 struct http *HTTP_create(void *p, uint16_t nhttp, unsigned);
 const char *http_Status2Reason(unsigned, const char **);
+int http_IsHdr(const txt *hh, hdr_t hdr);
 unsigned http_EstimateWS(const struct http *fm, unsigned how);
 void http_PutResponse(struct http *to, const char *proto, uint16_t status,
     const char *response);
@@ -590,9 +606,11 @@ void http_FilterReq(struct http *to, const struct http *fm, unsigned how);
 void HTTP_Encode(const struct http *fm, uint8_t *, unsigned len, unsigned how);
 int HTTP_Decode(struct http *to, const uint8_t *fm);
 void http_ForceHeader(struct http *to, hdr_t, const char *val);
+void http_AppendHeader(struct http *to, hdr_t, const char *val);
 void http_PrintfHeader(struct http *to, const char *fmt, ...)
     v_printflike_(2, 3);
 void http_TimeHeader(struct http *to, const char *fmt, vtim_real now);
+const char * http_ViaHeader(void);
 void http_Proto(struct http *to);
 void http_SetHeader(struct http *to, const char *header);
 void http_SetH(struct http *to, unsigned n, const char *header);
@@ -625,7 +643,7 @@ int HTTP_IterHdrPack(struct worker *, struct objcore *, const char **);
 #define HTTP_FOREACH_PACK(wrk, oc, ptr) \
 	 for ((ptr) = NULL; HTTP_IterHdrPack(wrk, oc, &(ptr));)
 const char *HTTP_GetHdrPack(struct worker *, struct objcore *, hdr_t);
-enum sess_close http_DoConnection(struct http *hp, enum sess_close sc_close);
+stream_close_t http_DoConnection(struct http *hp, stream_close_t sc_close);
 int http_IsFiltered(const struct http *hp, unsigned u, unsigned how);
 
 #define HTTPH_R_PASS		(1 << 0)	/* Request (c->b) in pass mode */
@@ -685,10 +703,13 @@ int Lck__Trylock(struct lock *lck, const char *p,  int l);
 void Lck__New(struct lock *lck, struct VSC_lck *, const char *);
 int Lck__Held(const struct lock *lck);
 int Lck__Owned(const struct lock *lck);
+extern pthread_mutexattr_t mtxattr_errorcheck;
 
 /* public interface: */
 void Lck_Delete(struct lock *lck);
-int Lck_CondWait(pthread_cond_t *cond, struct lock *lck, vtim_real);
+int Lck_CondWaitUntil(pthread_cond_t *, struct lock *, vtim_real when);
+int Lck_CondWait(pthread_cond_t *, struct lock *);
+int Lck_CondWaitTimeout(pthread_cond_t *, struct lock *, vtim_dur timeout);
 
 #define Lck_New(a, b) Lck__New(a, b, #b)
 #define Lck_Lock(a) Lck__Lock(a, __func__, __LINE__)
