@@ -36,12 +36,19 @@
 
 /*lint -save -e525 -e539 -e835 */
 
+#ifndef PARAM_ALL
+#  define PARAM_PRE
+#  define PARAM_POST
+#endif
+
 /*--------------------------------------------------------------------
  * Simple parameters
  */
 
-#define PARAM_SIMPLE(nm, typ, ...) \
-	PARAM(typ, nm, nm, tweak_##typ, &mgt_param.nm, __VA_ARGS__)
+#define PARAM_SIMPLE(nm, typ, ...)					\
+	PARAM_PRE							\
+	PARAM(typ, nm, nm, tweak_##typ, &mgt_param.nm, __VA_ARGS__)	\
+	PARAM_POST
 
 #if defined(PLATFORM_FLAGS)
 #  error "Temporary macro PLATFORM_FLAGS already defined"
@@ -385,13 +392,16 @@ PARAM_SIMPLE(
 	/* type */	timeout,
 	/* min */	"0.000",
 	/* max */	NULL,
-	/* def */	"10.000",
+	/* def */	"10s",
 	/* units */	"seconds",
 	/* descr */
 	"Default grace period.  We will deliver an object this long after "
 	"it has expired, provided another thread is attempting to get a "
 	"new copy.",
-	/* flags */	OBJ_STICKY
+	/* flags */	OBJ_STICKY,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"10s"
 )
 
 PARAM_SIMPLE(
@@ -399,14 +409,17 @@ PARAM_SIMPLE(
 	/* type */	timeout,
 	/* min */	"0.000",
 	/* max */	NULL,
-	/* def */	"0.000",
+	/* def */	"0s",
 	/* units */	"seconds",
 	/* descr */
 	"Default keep period.  We will keep a useless object around this "
 	"long, making it available for conditional backend fetches.  That "
 	"means that the object will be removed from the cache at the end "
 	"of ttl+grace+keep.",
-	/* flags */	OBJ_STICKY
+	/* flags */	OBJ_STICKY,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"0s"
 )
 
 PARAM_SIMPLE(
@@ -414,12 +427,15 @@ PARAM_SIMPLE(
 	/* type */	timeout,
 	/* min */	"0.000",
 	/* max */	NULL,
-	/* def */	"120.000",
+	/* def */	"2m",
 	/* units */	"seconds",
 	/* descr */
 	"The TTL assigned to objects if neither the backend nor the VCL "
 	"code assigns one.",
-	/* flags */	OBJ_STICKY
+	/* flags */	OBJ_STICKY,
+	/* dyn_min_reason */	NULL,
+	/* dyn_max_reason */	NULL,
+	/* dyn_def_reason */	"2m"
 )
 
 PARAM_SIMPLE(
@@ -855,7 +871,7 @@ PARAM_SIMPLE(
 	/* name */	tcp_keepalive_intvl,
 	/* type */	timeout,
 	/* min */	"1",
-	/* max */	"150",
+	/* max */	"100",
 	/* def */	NULL,
 	/* units */	"seconds",
 	/* descr */
@@ -888,7 +904,7 @@ PARAM_SIMPLE(
 	/* name */	tcp_keepalive_time,
 	/* type */	timeout,
 	/* min */	"1",
-	/* max */	"14400",
+	/* max */	"7200",
 	/* def */	NULL,
 	/* units */	"seconds",
 	/* descr */
@@ -1207,13 +1223,15 @@ PARAM_SIMPLE(
  */
 
 #define PARAM_MEMPOOL(nm, def, descr)					\
+	PARAM_PRE							\
 	PARAM(poolparam, nm, nm, tweak_poolparam, &mgt_param.nm,	\
 	    NULL, NULL, def, NULL,					\
 	    descr							\
 	    "The three numbers are:\n"					\
 	    "\tmin_pool\tminimum size of free pool.\n"			\
 	    "\tmax_pool\tmaximum size of free pool.\n"			\
-	    "\tmax_age\tmax age of free element.")
+	    "\tmax_age\tmax age of free element.")			\
+	PARAM_POST
 
 PARAM_MEMPOOL(
 		/* name */	pool_req,
@@ -1241,8 +1259,10 @@ PARAM_MEMPOOL(
  */
 
 #define PARAM_THREAD(nm, fld, typ, ...)			\
+	PARAM_PRE					\
 	PARAM(typ, wthread_ ## fld, nm, tweak_ ## typ,	\
-	    &mgt_param.wthread_ ## fld, __VA_ARGS__)
+	    &mgt_param.wthread_ ## fld, __VA_ARGS__)	\
+	PARAM_POST
 
 PARAM_THREAD(
 	/* name */	thread_pools,
@@ -1292,7 +1312,7 @@ PARAM_THREAD(
 	/* name */	thread_pool_min,
 	/* field */	min,
 	/* type */	thread_pool_min,
-	/* min */	"5" /* TASK_QUEUE__END */,
+	/* min */	"5" /* TASK_QUEUE_RESERVE */,
 	/* max */	NULL,
 	/* def */	"100",
 	/* units */	"threads",
@@ -1303,9 +1323,9 @@ PARAM_THREAD(
 	"situations or when threads have expired.\n"
 	"\n"
 	"Technical minimum is 5 threads, but this parameter is "
-	/*                    ^ TASK_QUEUE__END */
+	/*                    ^ TASK_QUEUE_RESERVE */
 	"strongly recommended to be at least 10",
-	/*               2 * TASK_QUEUE__END ^^ */
+	/*            2 * TASK_QUEUE_RESERVE ^^ */
 	/* flags */	DELAYED_EFFECT,
 	/* dyn_min_reason */	NULL,
 	/* dyn_max_reason */	"thread_pool_max"
@@ -1330,7 +1350,7 @@ PARAM_THREAD(
 	"priority tasks from running even under high load.\n"
 	"\n"
 	"The effective value is at least 5 (the number of internal "
-	/*                               ^ TASK_QUEUE__END */
+	/*                               ^ TASK_QUEUE_RESERVE */
 	"priority classes), irrespective of this parameter.",
 	/* flags */	DELAYED_EFFECT,
 	/* dyn_min_reason */	NULL,
@@ -1521,8 +1541,10 @@ PARAM_THREAD(
  * String parameters
  */
 
-#  define PARAM_STRING(nm, tw, pv, def, ...) \
-	PARAM(, , nm, tw, pv, NULL, NULL, def, NULL, __VA_ARGS__)
+#  define PARAM_STRING(nm, tw, pv, def, ...)				\
+	PARAM_PRE							\
+	PARAM(, , nm, tw, pv, NULL, NULL, def, NULL, __VA_ARGS__)	\
+	PARAM_POST
 
 PARAM_STRING(
 	/* name */	cc_command,
@@ -1612,42 +1634,15 @@ PARAM_STRING(
 	/* dyn_def_reason */	"${libdir}/varnish/vmods"
 )
 
-/*--------------------------------------------------------------------
- * VCC parameters
- */
-
-#  define PARAM_VCC(nm, def, descr) \
-	PARAM(, , nm, tweak_boolean, &mgt_ ## nm, NULL, NULL, def, "bool", descr)
-
-PARAM_VCC(
-	/* name */	vcc_err_unref,
-	/* def */	"on",
-	/* descr */
-	"Unreferenced VCL objects result in error."
-)
-
-PARAM_VCC(
-	/* name */	vcc_allow_inline_c,
-	/* def */	"off",
-	/* descr */
-	"Allow inline C code in VCL."
-)
-
-PARAM_VCC(
-	/* name */	vcc_unsafe_path,
-	/* def */	"on",
-	/* descr */
-	"Allow '/' in vmod & include paths.\n"
-	"Allow 'import ... from ...'."
-)
-
 /*--------------------------------------------------------------------
  * PCRE2 parameters
  */
 
 #  define PARAM_PCRE2(nm, pv, min, def, descr)			\
+	PARAM_PRE						\
 	PARAM(, , nm, tweak_uint, &mgt_param.vre_limits.pv,	\
-	    min, NULL, def, NULL, descr)
+	    min, NULL, def, NULL, descr)			\
+	PARAM_POST
 
 PARAM_PCRE2(
 	/* name */	pcre2_match_limit,
@@ -1690,107 +1685,128 @@ PARAM_PCRE2(
  * The deprecated_dummy alias is here for test coverage.
  */
 
-#define PARAM_ALIAS(al, nm) \
+#define PARAM_ALIAS(al, nm)					\
+	PARAM_PRE						\
 	PARAM(, , al, tweak_alias, NULL, NULL, NULL, #nm, NULL, \
-	    "Deprecated alias for the " #nm " parameter.")
+	    "Deprecated alias for the " #nm " parameter.")	\
+	PARAM_POST
 
-PARAM_ALIAS(deprecated_dummy, debug)
+PARAM_ALIAS(deprecated_dummy,	debug)
+PARAM_ALIAS(vcc_err_unref,	vcc_feature)
+PARAM_ALIAS(vcc_allow_inline_c,	vcc_feature)
+PARAM_ALIAS(vcc_unsafe_path,	vcc_feature)
 
 #  undef PARAM_ALIAS
-#  undef PARAM_ALL
 #  undef PARAM_PCRE2
 #  undef PARAM_STRING
-#  undef PARAM_VCC
 #endif /* defined(PARAM_ALL) */
 
+/*--------------------------------------------------------------------
+ * Bits parameters
+ */
+
+#define PARAM_BITS(nm, fld, def, descr)					\
+	PARAM(nm, fld, nm, tweak_ ## nm, mgt_param.fld, NULL, NULL,	\
+	    def, NULL, descr)
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	debug,
+	/* fld */	debug_bits,
+	/* def */	"none",
+	/* descr */
+	"Enable/Disable various kinds of debugging.\n"
+	"\tnone\tDisable all debugging\n\n"
+	"Use +/- prefix to set/reset individual bits:")
+#ifdef PARAM_ALL
+#  define DEBUG_BIT(U, l, d) "\n\t" #l "\t" d
+#  include "tbl/debug_bits.h"
+#endif
+PARAM_POST
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	experimental,
+	/* fld */	experimental_bits,
+	/* def */	"none",
+	/* descr */
+	"Enable/Disable experimental features.\n"
+	"\tnone\tDisable all experimental features\n\n"
+	"Use +/- prefix to set/reset individual bits:")
+#ifdef PARAM_ALL
+#  define EXPERIMENTAL_BIT(U, l, d) "\n\t" #l "\t" d
+#  include "tbl/experimental_bits.h"
+#endif
+PARAM_POST
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	feature,
+	/* fld */	feature_bits,
+	/* def */	"+validate_headers",
+	/* descr */
+	"Enable/Disable various minor features.\n"
+	"\tdefault\tSet default value\n"
+	"\tnone\tDisable all features.\n\n"
+	"Use +/- prefix to enable/disable individual feature:")
+#ifdef PARAM_ALL
+#  define FEATURE_BIT(U, l, d) "\n\t" #l "\t" d
+#  include "tbl/feature_bits.h"
+#endif
+PARAM_POST
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	vcc_feature,
+	/* fld */	vcc_feature_bits,
+	/* def */
+	"+err_unref,"
+	"+unsafe_path",
+	/* descr */
+	"Enable/Disable various VCC behaviors.\n"
+	"\tdefault\tSet default value\n"
+	"\tnone\tDisable all behaviors.\n\n"
+	"Use +/- prefix to enable/disable individual behavior:")
+#ifdef PARAM_ALL
+#  define VCC_FEATURE_BIT(U, l, d) "\n\t" #l "\t" d
+#  include "tbl/vcc_feature_bits.h"
+#endif
+PARAM_POST
+
+PARAM_PRE
+PARAM_BITS(
+	/* name */	vsl_mask,
+	/* fld */	vsl_mask,
+	/* def */
+	"-Debug,"
+	"-ExpKill,"
+	"-H2RxBody,"
+	"-H2RxHdr,"
+	"-H2TxBody,"
+	"-H2TxHdr,"
+	"-Hash,"
+	"-ObjHeader,"
+	"-ObjProtocol,"
+	"-ObjReason,"
+	"-ObjStatus,"
+	"-VCL_trace,"
+	"-VdpAcct,"
+	"-VfpAcct,"
+	"-WorkThread",
+	/* descr */
+	"Mask individual VSL messages from being logged.\n"
+	"\tdefault\tSet default value\n"
+	"\nUse +/- prefix in front of VSL tag name to unmask/mask "
+	"individual VSL messages.")
+PARAM_POST
+
+#undef PARAM_ALL
+#undef PARAM_BITS
 #undef PARAM_MEMPOOL
+#undef PARAM_POST
+#undef PARAM_PRE
 #undef PARAM_SIMPLE
 #undef PARAM_THREAD
 #undef PARAM
 
-#if 0 /* NOT ACTUALLY DEFINED HERE */
-/* actual location mgt_param_bits.c*/
-/* see tbl/debug_bits.h */
-PARAM(
-	/* name */	debug,
-	/* type */	debug,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* def */	NULL,
-	/* units */	NULL,
-	/* descr */
-	"Enable/Disable various kinds of debugging.\n"
-	"	none	Disable all debugging\n"
-	"\n"
-	"Use +/- prefix to set/reset individual bits:\n"
-	"	req_state	VSL Request state engine\n"
-	"	workspace	VSL Workspace operations\n"
-	"	waiter	VSL Waiter internals\n"
-	"	waitinglist	VSL Waitinglist events\n"
-	"	syncvsl	Make VSL synchronous\n"
-	"	hashedge	Edge cases in Hash\n"
-	"	vclrel	Rapid VCL release\n"
-	"	lurker	VSL Ban lurker\n"
-	"	esi_chop	Chop ESI fetch to bits\n"
-	"	flush_head	Flush after http1 head\n"
-	"	vtc_mode	Varnishtest Mode"
-)
-
-/* actual location mgt_param_bits.c*/
-/* see tbl/experimental_bits.h */
-PARAM(
-	/* name */	experimental,
-	/* type */	experimental,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* def */	NULL,
-	/* units */	NULL,
-	/* descr */
-	"Enable/Disable experimental features.\n"
-	"	none	Disable all experimental features\n"
-	"\n"
-	"Use +/- prefix to set/reset individual bits:\n"
-	"	drop_pools	Drop thread pools\n"
-)
-
-/* actual location mgt_param_bits.c*/
-/* See tbl/feature_bits.h */
-PARAM(
-	/* name */	feature,
-	/* type */	feature,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* def */	NULL,
-	/* units */	NULL,
-	/* descr */
-	"Enable/Disable various minor features.\n"
-	"	none	Disable all features.\n"
-	"\n"
-	"Use +/- prefix to enable/disable individual feature:\n"
-	"	short_panic	Short panic message.\n"
-	"	wait_silo	Wait for persistent silo.\n"
-	"	no_coredump	No coredumps.\n"
-	"	esi_ignore_https	Treat HTTPS as HTTP in ESI:includes\n"
-	"	esi_disable_xml_check	Don't check of body looks like XML\n"
-	"	esi_ignore_other_elements	Ignore non-esi XML-elements\n"
-	"	esi_remove_bom	Remove UTF-8 BOM"
-)
-
-/* actual location mgt_param_bits.c*/
-PARAM(
-	/* name */	vsl_mask,
-	/* type */	vsl_mask,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* def */	"default",
-	/* units */	NULL,
-	/* descr */
-	"Mask individual VSL messages from being logged.\n"
-	"	default	Set default value\n"
-	"\n"
-	"Use +/- prefix in front of VSL tag name to unmask/mask "
-	"individual VSL messages."
-)
-#endif /* NOT ACTUALLY DEFINED HERE */
-
 /*lint -restore */
