@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -38,7 +40,11 @@
 #ifndef VAS_H_INCLUDED
 #define VAS_H_INCLUDED
 
-#include "verrno.h"
+
+#include <errno.h>
+#include <stddef.h>	// size_t
+
+const char * VAS_errtxt(int e);
 
 enum vas_e {
 	VAS_WRONG,
@@ -112,4 +118,19 @@ do {									\
 		typedef char __vassert_## y[(x) ? 1 : -1] v_unused_
 #endif
 
+/*
+ * A normal pointer difference is signed, but when we don't want a negative
+ * value this little tool will make sure we don't get that.
+ */
+
+static inline size_t
+pdiff(const void *b, const void *e)
+{
+
+	AN(b);
+	AN(e);
+	assert(b <= e);
+	return ((size_t)((const char *)e - (const char *)b));
+}
+
 #endif
