@@ -1,7 +1,7 @@
 /*-
  * SPDX-License-Identifier: BSD-2-Clause-FreeBSD
  *
- * Copyright (c) 2000-2011 Poul-Henning Kamp
+ * Copyright (c) 2000-2011 Poul-Henning Kamp <phk@FreeBSD.org>
  * Copyright (c) 2000-2008 Dag-Erling Coïdan Smørgrav
  * All rights reserved.
  *
@@ -59,9 +59,8 @@ extern "C" {
 /*
  * API functions
  */
-struct vsb	*VSB_new(struct vsb *, char *, int, int);
-#define		 VSB_new_auto()				\
-	VSB_new(NULL, NULL, 0, VSB_AUTOEXTEND)
+struct vsb	*VSB_init(struct vsb *, void *, ssize_t);
+struct vsb	*VSB_new_auto(void);
 void		 VSB_clear(struct vsb *);
 int		 VSB_bcat(struct vsb *, const void *, ssize_t);
 int		 VSB_cat(struct vsb *, const char *);
@@ -76,15 +75,54 @@ int		 VSB_error(const struct vsb *);
 int		 VSB_finish(struct vsb *);
 char		*VSB_data(const struct vsb *);
 ssize_t		 VSB_len(const struct vsb *);
-void		 VSB_delete(struct vsb *);
+void		 VSB_fini(struct vsb *);
 void		 VSB_destroy(struct vsb **);
-#define VSB_QUOTE_NONL		1
+
+/*
+ * VSB_quote[_pfx] has four major modes, and two modifiers
+ */
+
+#define VSB_QUOTE_PLAIN		0
+	/*
+	 * Basic "show me the string" mode
+	 * All output is a single line
+	 */
 #define VSB_QUOTE_JSON		2
+	/*
+	 * Output suitable for inclusion between "..." in JSON
+	 * Uses JSON \u%04x quoting.
+	 * Anything above 0x7e had better be UTF-8
+	 */
 #define VSB_QUOTE_HEX		4
+	/*
+	 * Hex dump, single line.
+	 * All zero data is compressed to "0x0...0"
+	 */
 #define VSB_QUOTE_CSTR		8
+	/*
+	 * C lanuage source code literal string(s)
+	 * Breaks strings at \n (expecting string concatenation)
+	 */
 #define VSB_QUOTE_UNSAFE	16
+	/*
+	 * For general display applications
+	 * " and \ are not quoted
+	 * Splits output to new line at '\n'
+	 * Implies VSB_QUOTE_NONL
+	 */
+
+#define VSB_QUOTE_NONL		1
+	/*
+	 * If the output does not end in \n, append \n
+	 * Can be combined with all other modes.
+	 */
+
 #define VSB_QUOTE_ESCHEX	32
-#define VSB_QUOTE_GLOB		64
+	/*
+	 * Use \x%02x instead of \%03o
+	 * Not valid with VSB_QUOTE_JSON and VSB_QUOTE_HEX
+	 */
+
 void		 VSB_quote_pfx(struct vsb *, const char*, const void *,
 		     int len, int how);
 void		 VSB_quote(struct vsb *, const void *, int len, int how);
